import { Component, Input, ViewChild, Renderer2, ViewEncapsulation, NgModule } from '@angular/core';
import { BrowserModule } from '@angular/platform-browser';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {number} */
const Separator = {
    NA: 0,
    Big: 1,
    Small: 2,
};
Separator[Separator.NA] = 'NA';
Separator[Separator.Big] = 'Big';
Separator[Separator.Small] = 'Small';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const Config = {
    WIDTH: 200,
    ARC_STROKE: 5,
    ARROW_Y: 22.5,
    ARROW_WIDTH: 4,
    ARROW_PIN_RAD: 8,
    SL_NORM: 3,
    SL_MID_SEP: 7,
    SL_SEP: 10,
    SL_WIDTH: 2,
    TXT_MARGIN: 10,
    LIGHT_Y: 55,
    LIGHT_RADIUS: 10,
    S_FAC_Y: 80,
    DIGITAL_Y: 145,
    UNIT_Y: 155,
    MAX_PURE_SCALE_VAL: 1000,
    INIT_LINE_FREQ: 2,
    DEF_START: 225,
    DEF_END: 135
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const showError = (text) => {
    console.error(`GaugeComponent: ${text}`);
};
/** @type {?} */
const validate = (props) => {
    if (!props.max) {
        showError('The maximal value is not set.');
    }
    if (!(0 <= props.start && props.start <= 359) || !(0 <= props.end && props.end <= 359)) {
        showError('The end and start must be between 0 and 359 degrees.');
    }
    if (props.light && props.light > props.max) {
        showError('The red light trigger value cannot be greater than the max value of the gauge.');
    }
    if (props.factor && props.factor >= props.max) {
        showError('The factor cannot be greater than or equal to the max value.');
    }
    if (props.sectors) {
        props.sectors.forEach((s) => {
            if (s.from > props.max || s.to > props.max) {
                showError('The sector bounds cannot be greater than the max value.');
            }
            if (s.from >= s.to) {
                showError('The lower bound of the sector cannot be greater than or equal to the upper one.');
            }
            if (!s.color) {
                showError(`Sector[${s.from}, ${s.to}] color is empty.`);
            }
        });
    }
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class Ng2GaugeComponent {
    /**
     * @param {?} _renderer
     */
    constructor(_renderer) {
        this._renderer = _renderer;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set input(val) {
        this._input = val;
        this._updateArrowPos(val);
    }
    /**
     * @return {?}
     */
    get input() {
        return this._input;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set max(val) {
        if (this._max) {
            this._max = val;
            this._initialize();
        }
        this._max = val;
    }
    /**
     * @return {?}
     */
    get max() {
        return this._max;
    }
    /**
     * @return {?}
     */
    get arc() {
        return this._arc(0, this._end);
    }
    /**
     * @return {?}
     */
    get gaugeRotationAngle() {
        return this._end - this.end;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.config = Object.assign(Config, this.config);
        if (!this.start) {
            this.start = this.config.DEF_START;
        }
        if (!this.end) {
            this.end = this.config.DEF_END;
        }
        validate(this);
        /** @type {?} */
        const width = this.config.WIDTH + this.config.ARC_STROKE;
        this.viewBox = `0 0 ${width} ${width}`;
        this.radius = this.config.WIDTH / 2;
        this.center = width / 2;
        this._end = this.end;
        if (this.start > this.end) {
            this._end += (360 - this.start);
        }
        else {
            this._end -= this.start;
        }
        this._initialize();
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this._rotateGauge();
    }
    /**
     * Initialize gauge.
     * @private
     * @return {?}
     */
    _initialize() {
        this.scaleLines = [];
        this.scaleValues = [];
        this._calculateSectors();
        this._updateArrowPos(this._input);
        this.scaleFactor = this.factor || this._determineScaleFactor();
        this._createScale();
    }
    /**
     * Calculate arc.
     * @private
     * @param {?} start
     * @param {?} end
     * @return {?}
     */
    _arc(start, end) {
        /** @type {?} */
        const largeArc = end - start <= 180 ? 0 : 1;
        /** @type {?} */
        const startCoor = this._getAngleCoor(start);
        /** @type {?} */
        const endCoor = this._getAngleCoor(end);
        return `M ${endCoor.x} ${endCoor.y} A ${this.radius} ${this.radius} 0 ${largeArc} 0 ${startCoor.x} ${startCoor.y}`;
    }
    /**
     * Get angle coordinates (Cartesian coordinates).
     * @private
     * @param {?} degrees
     * @return {?}
     */
    _getAngleCoor(degrees) {
        /** @type {?} */
        const rads = (degrees - 90) * Math.PI / 180;
        return {
            x: (this.radius * Math.cos(rads)) + this.center,
            y: (this.radius * Math.sin(rads)) + this.center
        };
    }
    /**
     * Calculate/translate the user-defined sectors to arcs.
     * @private
     * @return {?}
     */
    _calculateSectors() {
        if (!this.sectors) {
            return;
        }
        this._mappedSectors = JSON.parse(JSON.stringify(this.sectors));
        this._mappedSectors.forEach((s) => {
            /** @type {?} */
            const ratio = this._end / this.max;
            s.from *= ratio;
            s.to *= ratio;
        });
        this.sectorArcs = this._mappedSectors.map((s) => {
            return {
                path: this._arc(s.from, s.to),
                color: s.color
            };
        });
    }
    /**
     * Update the position of the arrow based on the input.
     * @private
     * @param {?} input
     * @return {?}
     */
    _updateArrowPos(input) {
        /** @type {?} */
        const pos = (this._end / this.max) * input;
        this._renderer.setStyle(this.arrow.nativeElement, 'transform', `rotate(${pos}deg)`);
    }
    /**
     * Rotate the gauge based on the start property. The CSS rotation, saves additional calculations with SVG.
     * @private
     * @return {?}
     */
    _rotateGauge() {
        /** @type {?} */
        const angle = 360 - this.start;
        this._renderer.setStyle(this.gauge.nativeElement, 'transform', `rotate(-${angle}deg)`);
    }
    /**
     * Determine the scale factor (10^n number; i.e. if max = 9000 then scale_factor = 1000)
     * @private
     * @param {?=} factor
     * @return {?}
     */
    _determineScaleFactor(factor = 10) {
        // Keep smaller factor until 3X
        if (this.max / factor > 30) {
            return this._determineScaleFactor(factor * 10);
        }
        return factor;
    }
    /**
     * Determine the line frequency which represents after what angle we should put a line.
     * @private
     * @return {?}
     */
    _determineLineFrequency() {
        /** @type {?} */
        const separators = this.max / this.scaleFactor;
        /** @type {?} */
        const separateAtAngle = this._end / separators;
        /** @type {?} */
        let lineFrequency;
        // If separateAtAngle is not an integer, use its value as the line frequency.
        if (separateAtAngle % 1 !== 0) {
            lineFrequency = separateAtAngle;
        }
        else {
            lineFrequency = this.config.INIT_LINE_FREQ * 2;
            for (lineFrequency; lineFrequency <= separateAtAngle; lineFrequency++) {
                if (separateAtAngle % lineFrequency === 0) {
                    break;
                }
            }
        }
        return lineFrequency;
    }
    /**
     * Checks whether the line (based on index) is big or small separator.
     * @private
     * @param {?} idx
     * @param {?} lineFrequency
     * @return {?}
     */
    _isSeparatorReached(idx, lineFrequency) {
        /** @type {?} */
        const separators = this.max / this.scaleFactor;
        /** @type {?} */
        const totalSeparators = this._end / lineFrequency;
        /** @type {?} */
        const separateAtIdx = totalSeparators / separators;
        if (idx % separateAtIdx === 0) {
            return Separator.Big;
        }
        else if (idx % (separateAtIdx / 2) === 0) {
            return Separator.Small;
        }
        return Separator.NA;
    }
    /**
     * Creates the scale.
     * @private
     * @return {?}
     */
    _createScale() {
        /** @type {?} */
        const accumWith = this._determineLineFrequency() / 2;
        /** @type {?} */
        const isAboveSuitableFactor = this.max / this.scaleFactor > 10;
        /** @type {?} */
        let placedVals = 0;
        for (let alpha = 0, i = 0; alpha >= (-1) * this._end; alpha -= accumWith, i++) {
            /** @type {?} */
            let lineHeight = this.config.SL_NORM;
            /** @type {?} */
            const sepReached = this._isSeparatorReached(i, accumWith);
            // Set the line height based on its type
            switch (sepReached) {
                case Separator.Big:
                    placedVals++;
                    lineHeight = this.config.SL_SEP;
                    break;
                case Separator.Small:
                    lineHeight = this.config.SL_MID_SEP;
                    break;
            }
            // Draw the line
            /** @type {?} */
            const higherEnd = this.center - this.config.ARC_STROKE - 2;
            /** @type {?} */
            const lowerEnd = higherEnd - lineHeight;
            /** @type {?} */
            const alphaRad = Math.PI / 180 * (alpha + 180);
            /** @type {?} */
            const sin = Math.sin(alphaRad);
            /** @type {?} */
            const cos = Math.cos(alphaRad);
            /** @type {?} */
            const color = this._getScaleLineColor(alpha);
            this._addScaleLine(sin, cos, higherEnd, lowerEnd, color);
            // Put a scale value
            if (sepReached === Separator.Big) {
                /** @type {?} */
                const isValuePosEven = placedVals % 2 === 0;
                /** @type {?} */
                const isLast = alpha <= (-1) * this._end;
                if (!(isAboveSuitableFactor && isValuePosEven && !isLast)) {
                    this._addScaleValue(sin, cos, lowerEnd, alpha);
                }
            }
        }
    }
    /**
     * Get the scale line color from the user-provided sectors definitions.
     * @private
     * @param {?} alpha
     * @return {?}
     */
    _getScaleLineColor(alpha) {
        alpha *= (-1);
        /** @type {?} */
        let color = '';
        if (this._mappedSectors) {
            this._mappedSectors.forEach((s) => {
                if (s.from <= alpha && alpha <= s.to) {
                    color = s.color;
                }
            });
        }
        return color;
    }
    /**
     * Add a scale line to the list that will be later rendered.
     * @private
     * @param {?} sin
     * @param {?} cos
     * @param {?} higherEnd
     * @param {?} lowerEnd
     * @param {?} color
     * @return {?}
     */
    _addScaleLine(sin, cos, higherEnd, lowerEnd, color) {
        this.scaleLines.push({
            from: {
                x: sin * higherEnd + this.center,
                y: cos * higherEnd + this.center
            },
            to: {
                x: sin * lowerEnd + this.center,
                y: cos * lowerEnd + this.center
            },
            color
        });
    }
    /**
     * Add a scale value.
     * @private
     * @param {?} sin
     * @param {?} cos
     * @param {?} lowerEnd
     * @param {?} alpha
     * @return {?}
     */
    _addScaleValue(sin, cos, lowerEnd, alpha) {
        /** @type {?} */
        let val = Math.round(alpha * (this.max / this._end)) * (-1);
        /** @type {?} */
        let posMargin = this.config.TXT_MARGIN * 2;
        // Use the multiplier instead of the real value, if above MAX_PURE_SCALE_VAL (i.e. 1000)
        if (this.max > this.config.MAX_PURE_SCALE_VAL) {
            val /= this.scaleFactor;
            val = Math.round(val * 100) / 100;
            posMargin /= 2;
        }
        this.scaleValues.push({
            text: val.toString(),
            coor: {
                x: sin * (lowerEnd - posMargin) + this.center,
                y: cos * (lowerEnd - posMargin) + this.center
            }
        });
    }
}
Ng2GaugeComponent.decorators = [
    { type: Component, args: [{
                selector: 'nga-ng2-gauge',
                template: "<section class=\"ng-gauge\" [class.light]=\"lightTheme\" [style.width]=\"size + 'px'\">\n  <svg class=\"info\" [attr.viewBox]=\"viewBox\" xmlns=\"http://www.w3.org/2000/svg\">\n    <circle *ngIf=\"light\"  class=\"red-light\"\n      [class.on]=\"input >= light\"\n      [attr.cx]=\"center\"\n      [attr.cy]=\"config.LIGHT_Y\"\n      [attr.r]=\"config.LIGHT_RADIUS\">\n    </circle>\n    <text *ngIf=\"max > config.MAX_PURE_SCALE_VAL\"\n      class=\"factor\"\n      [attr.x]=\"center\"\n      [attr.y]=\"config.S_FAC_Y\">\n      x{{scaleFactor}} {{unit}}\n    </text>\n    <text *ngIf=\"showDigital\"\n      class=\"digital\"\n      [attr.x]=\"center\"\n      [attr.y]=\"config.DIGITAL_Y\">\n      {{input}}\n    </text>\n    <text class=\"unit\" [attr.x]=\"center\" [attr.y]=\"config.UNIT_Y\">{{unit}}</text>\n  </svg>\n  <svg #gauge [attr.viewBox]=\"viewBox\" xmlns=\"http://www.w3.org/2000/svg\">\n    <path class=\"main-arc\" [attr.d]=\"arc\" [attr.stroke-width]=\"config.ARC_STROKE\" fill=\"none\" />\n    <path *ngFor=\"let arc of sectorArcs\"\n      [attr.d]=\"arc.path\"\n      [attr.stroke]=\"arc.color\"\n      [attr.stroke-width]=\"config.ARC_STROKE\"\n      fill=\"none\" />\n    <line *ngFor=\"let line of scaleLines\"\n      [attr.stroke-width]=\"config.SL_WIDTH\"\n      [attr.stroke]=\"line.color || (!lightTheme ? '#333' : '#fff')\"\n      [attr.x1]=\"line.from.x\"\n      [attr.y1]=\"line.from.y\"\n      [attr.x2]=\"line.to.x\"\n      [attr.y2]=\"line.to.y\" />\n    <text *ngFor=\"let val of scaleValues\"\n      class=\"text-val\"\n      dominant-baseline=\"central\"\n      [attr.x]=\"val.coor.x\"\n      [attr.y]=\"val.coor.y\"\n      [attr.transform]=\"'rotate(' + gaugeRotationAngle + ', ' + val.coor.x + ', ' + val.coor.y + ')'\">\n      {{val.text}}\n    </text>\n    <rect #arrow\n      class=\"arrow\"\n      [attr.x]=\"center - config.ARROW_WIDTH / 2\"\n      [attr.y]=\"config.ARROW_Y\"\n      [attr.height]=\"center - config.ARROW_Y\"\n      [attr.width]=\"config.ARROW_WIDTH\"\n      [attr.rx]=\"config.ARROW_WIDTH / 2\"\n      [attr.ry]=\"config.ARROW_WIDTH / 2\">\n    </rect>\n    <circle class=\"arrow-pin\" [attr.cx]=\"center\" [attr.cy]=\"center\" [attr.r]=\"config.ARROW_PIN_RAD\" />\n  </svg>\n</section>\n",
                encapsulation: ViewEncapsulation.None,
                styles: ["@font-face{font-family:Orbitron;font-style:normal;font-weight:700;src:local('Orbitron Bold'),local('Orbitron-Bold'),url(https://fonts.gstatic.com/s/orbitron/v8/Y82YH_MJJWnsH2yUA5AuYY4P5ICox8Kq3LLUNMylGO4.woff2) format('woff2');unicode-range:U+0000-00FF,U+0131,U+0152-0153,U+02C6,U+02DA,U+02DC,U+2000-206F,U+2074,U+20AC,U+2212,U+2215}.ng-gauge{position:relative;width:400px}.ng-gauge svg.info{position:absolute;top:0;left:0}.ng-gauge rect.arrow{-webkit-transform-origin:50% 50%;transform-origin:50% 50%;fill:orange}.ng-gauge text{font-family:Orbitron,sans-serif;font-weight:700;text-anchor:middle;fill:#333}.ng-gauge.light text{fill:#fff}.ng-gauge text.text-val{font-size:12px}.ng-gauge circle.arrow-pin{fill:#333}.ng-gauge path.main-arc{stroke:#333}.ng-gauge.light path.main-arc{stroke:#fff}.ng-gauge text.factor{font-size:7px}.ng-gauge text.digital{font-size:16px}.ng-gauge text.unit{font-size:10px}.ng-gauge circle.red-light{fill:#ff4f4f;opacity:.1}.ng-gauge circle.red-light.on{opacity:1}"]
            }] }
];
/** @nocollapse */
Ng2GaugeComponent.ctorParameters = () => [
    { type: Renderer2 }
];
Ng2GaugeComponent.propDecorators = {
    gauge: [{ type: ViewChild, args: ['gauge',] }],
    arrow: [{ type: ViewChild, args: ['arrow',] }],
    size: [{ type: Input }],
    start: [{ type: Input }],
    end: [{ type: Input }],
    sectors: [{ type: Input }],
    unit: [{ type: Input }],
    showDigital: [{ type: Input }],
    light: [{ type: Input }],
    lightTheme: [{ type: Input }],
    factor: [{ type: Input }],
    config: [{ type: Input }],
    input: [{ type: Input }],
    max: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class Ng2GaugeModule {
}
Ng2GaugeModule.decorators = [
    { type: NgModule, args: [{
                declarations: [Ng2GaugeComponent],
                imports: [
                    BrowserModule
                ],
                exports: [Ng2GaugeComponent]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { Ng2GaugeComponent, Ng2GaugeModule };

//# sourceMappingURL=ng2-gauge.js.map